/************************************************************************/
/*                                                                      */
/*                      Motorpotiplatine Steuerung                      */
/*                                                                      */
/*              Author : Frank Link                                     */
/*                       franklink61@aol.com                            */
/*              Version: 1.0                                            */
/************************************************************************/
#include "main.h"
#include <util\delay.h>


#ifndef MOTORHEADER
#include "motor.h"
#endif

#ifndef TIMERHEADER
#include "timer.h"
#endif

#ifndef BUTTONSHEADER
#include "buttons.h"
#endif

// #define debug

/******************************	UART Ausgabe *****************************/

#ifdef debug

void putchar( char c )
{
  while( (UCSRA & 1<<UDRE) == 0 );
  UDR = c;
}


void puts( char *s )
{
  while( *s )
    putchar( *s++ );
}

#endif

/******************************	Interrupts *******************************/

/*
	Hochsetzen des Timers und gegebenenfalls stoppen des Motors
*/
ISR(TIMER1_OVF_vect)
{
  timerHighByte++;
  if ( ( timerHighByte >= MOTOR_ON_TIME ) &&
       ( ( ( MotorRightIsOn == MOTOR_ON ) || ( MotorLeftIsOn != MOTOR_ON ) ) ) )
  {
	stopTimer();						// Der Timer wird ausgeschaltet
  	MotorOff();							// Der Motor wird ausgeschaltet
  }
}

/*
	RC5 Routine von Peter Dannegger
*/
ISR( TIMER0_OVF_vect )
{
  uint tmp = rc5_tmp;				// for faster access

  TCNT0 = -2;					// 2 * 256 = 512 cycle

  if( ++rc5_time > PULSE_MAX ){			// count pulse time
    if( !(tmp & 0x4000) && tmp & 0x2000 )	// only if 14 bits received
      rc5_data = tmp;
    tmp = 0;
  }

  if( (rc5_bit ^ xRC5_IN) & 1<<xRC5 ){		// change detect
    rc5_bit = ~rc5_bit;				// 0x00 -> 0xFF -> 0x00

    if( rc5_time < PULSE_MIN )			// to short
      tmp = 0;

    if( !tmp || rc5_time > PULSE_1_2 ){		// start or long pulse time
      if( !(tmp & 0x4000) )			// not to many bits
        tmp <<= 1;				// shift
      if( !(rc5_bit & 1<<xRC5) )		// inverted bit
        tmp |= 1;				// insert new bit
      rc5_time = 0;				// count next pulse time
    }
  }

  rc5_tmp = tmp;
  f_timer_tick = 1;
}

/******************************	Motor ************************************/

/*
	Motor fr Linkslauf einschalten
*/
void motor_left_on( void )
{
	stopTimer();						// Der Timer wird ausgeschaltet
	if ( MotorRightIsOn == MOTOR_ON )   // dreht der Motor in die andere Richtung
	{
		MotorOff();						// Motor ausschalten
	}
	if ( MotorLeftIsOn != MOTOR_ON )	// Dreht der Motor bereit in diese Richtung
	{
		MotorLeftOn();					// wenn nein, dann einschalten
	}
	restartTimer();						// Timer neu starten
}

/*
	Motor fr Rechtslauf einschalten
*/
void motor_right_on( void )
{
	stopTimer();						// Der Timer wird ausgeschaltet
	if ( MotorLeftIsOn == MOTOR_ON )    // dreht der Motor in die andere Richtung
	{
		MotorOff();						// Motor ausschalten
	}
	if ( MotorRightIsOn != MOTOR_ON )	// Dreht der Motor bereit in diese Richtung
	{
		MotorRightOn();					// wenn nein, dann einschalten
	}
	restartTimer();						// Timer neu starten
}

/*
	Alle Ports werden abgeschaltet
*/
void devices_off( void )
{
	DDRD  = ALL_OFF;
	PORTD = ALL_OFF;
	MotorOff();
}
/******************************	Main *****************************/
int main( void )
{
 	uint i;									// Variable mit gltigem RC5-Code
	uint key;								// Key-Code der gedrckten Taste
	uint device;							// Device-Code der gedrckten Taste
	int toggle;								// Toggle-Bit der gedrckten Taste
	int oldtoggle;							// vorheriger Zustand des Toggle-Bit
	uint ir_on;								// Zustand des Device 0 = ausgeschaltet / 1 = eingeschaltet
	int oldkey;	    						// Zuletzt verwendete Taste, dient zur Entprellung der Eingabe. 
											// Wird verwendet, um die Tasten Channel+ und Channel- zu entprellen
   	uint channel;       					// Zwischenspeicher fr den letzten Kanal. 
   											// Diese Variable wird im EEPROM gespeichert

#ifdef debug
	char s[30];								// Speicherbereich fr die Ausgabe von Zeichen ber UART
#endif
   	 
	TCCR0  = 1<<CS02;			         	// divide by 256
	TCCR1B = 1<<CS10;       			 	// divide by 256
  	TIMSK  = 1<<TOIE0^1<<TOIE1;	         	// enable timer interrupt
	stopTimer();
	
  	UBRRL = bauddivider;			     	// Baudrate
  	UBRRH = bauddivider >> 8;
  	UCSRA = 0;				             	
  	UCSRC = 1<<URSEL^1<<UCSZ1^1<<UCSZ0;	 	// 8 Bit
  	UCSRB = 1<<RXEN^1<<TXEN;		     	// enable RX, TX

  	key = 0x01;								// Initialisierung fr Key
  	oldkey = key;		
  	oldtoggle = -1;      					// Initialisierung fr Togglebit
  	
  	ir_on = IR_ON;							
  	DDRD  = CHANNEL1_INIT; 					// Bei Type1 wird automatisch Channel 1 eingeschaltet um sicherzustellen,
	PORTD = CHANNEL1_INIT;  				// dass die Platine in einem definierten Zustand ist.
											// Bei Type2 sind alle Ports ausgeschaltet und die Platine muss explizit
											// eingeschaltet werden
	channel = 1;  	    					// Kanal 1 soll aktiviert werden, wenn das Gert ber die Ein-Austaste eingeschaltet wird.
  	
  	sei();									// Interrupts global freigeben

  	/* Main-Loop */
  	for(;;)
  	{
    	cli();
    	i = rc5_data;						// lese die zwei Bytes fr die RC5-Codierung
    	rc5_data = 0;
    	sei();
    	if( i )
    	{
    	
			device = i >> 6 & 0x1F;			// Device-Code auslesen
			key = (i & 0x3F);				// Key-Code auslesen
			toggle = ( i >> 11 & 1);		// Toggle-Bit auslesen

			#ifdef debug
      			putchar(( i >> 11 & 1) + '0');				// Toggle Bit
      			putchar(' ');
      			itoa( i >> 6 & 0x1F, s, 10);				// Device address
      			puts( s );
      			putchar(' ');
      			itoa((i & 0x3F) | (~i >> 7 & 0x40), s, 10); // Key Code
      			puts( s );
      			putchar( 10 );
      			putchar( 13 );
			#endif    

			
			if ( ( isKey_on_off( device, key ) == 1 ) && 
			     ( ir_on == IR_OFF ) && ( oldtoggle != toggle ) )
			{
				#ifdef debug
					puts( "on" );
				#endif
				ir_on = IR_ON;
				oldtoggle = toggle;
	      		sendCode( channel );			
			} else
			if ( ( isKey_on_off( device, key ) == 1 ) && 
			     ( ir_on == IR_ON ) && ( oldtoggle != toggle ) )
			{
				#ifdef debug
					puts( "off" );
				#endif
				ir_on = IR_OFF;
				oldtoggle = toggle;
				devices_off();	
			}

			if ( ir_on == IR_ON )
			{
		      	if ( isKey_Motor_Left( device, key ) == 1 )
		      	{
					#ifdef debug
						puts( ">" );
					#endif
		      		motor_left_on();
		      	}
	    		if ( isKey_Motor_Right( device, key ) == 1 )
	    		{
					#ifdef debug
						puts( "<" );
					#endif
		      		motor_right_on();
		      	}
				if ( isKey_Channel_Up( device, key ) == 1 )
				{
					#ifdef debug
						puts( "+" );
					#endif
	      			if ( oldkey != i )
	      			{
	      				channel++;
	      				if ( channel > MAX_CHANNELS )
	      				{
	      					channel = 1;
	      				};
	      				channel = sendCode( channel );			
	      				oldkey = i;
	      			}
				}
				if ( isKey_Channel_Down( device, key ) == 1 )
				{
					#ifdef debug
						puts( "-" );
					#endif
	      			if ( oldkey != i )
	      			{
	      				channel--;
	      				if ( channel == 0 )
	      				{
	      					channel = MAX_CHANNELS;
	      				};
	      				channel = sendCode( channel );			
	      				oldkey = i;
	      			}
				}
			}
			if( isKey_1( device, key ) == 1 )
			{
				#ifdef debug
					puts( "1" );
				#endif
				if ( ir_on == IR_OFF )
				{
					ir_on = IR_ON;
					oldtoggle = -1;
				}
				channel = sendCode( 0x01 );
			}
			if( isKey_2( device, key ) == 1 )
			{
				#ifdef debug
					puts( "2" );
				#endif
				if ( ir_on == IR_OFF )
				{
					ir_on = IR_ON;
					oldtoggle = -1;
				}
				channel = sendCode( 0x02 );
			}
			if( isKey_3( device, key ) == 1 )
			{
				#ifdef debug
					puts( "3" );
				#endif
				if ( ir_on == IR_OFF )
				{
					ir_on = IR_ON;
					oldtoggle = -1;
				}
				channel = sendCode( 0x03 );
			}
			if( isKey_4( device, key ) == 1 )
			{
				#ifdef debug
					puts( "4" );
				#endif
				if ( ir_on == IR_OFF )
				{
					ir_on = IR_ON;
					oldtoggle = -1;
				}
				channel = sendCode( 0x04 );
			}
			if( isKey_5( device, key ) == 1 )
			{
				#ifdef debug
					puts( "5" );
				#endif
				if ( ir_on == IR_OFF )
				{
					ir_on = IR_ON;
					oldtoggle = -1;
				}
				channel = sendCode( 0x05 );
			}
		}
  	}
}

